@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
+
+#include <config.h>
 #include "dbus-print-message.h"
 
 #include <stdlib.h>
@@ -54,7 +56,7 @@ indent (int depth)
 static void
 print_hex (unsigned char *bytes, unsigned int len, int depth)
 {
-  int i, columns;
+  unsigned int i, columns;
 
   printf ("array of bytes [\n");
 
@@ -344,7 +346,15 @@ print_iter (DBusMessageIter *iter, dbus_bool_t literal, int depth)
 	    printf("}\n");
 	    break;
 	  }
-	    
+
+	case DBUS_TYPE_UNIX_FD:
+	  {
+	    int fd;
+	    dbus_message_iter_get_basic (iter, &fd);
+	    printf ("unix fd %d\n", fd);
+	    break;
+	  }
+
 	default:
 	  printf (" (dbus-monitor too dumb to decipher arg type '%c')\n", type);
 	  break;
